! ***********************************************************************
!
!   Copyright (C) 2010-2019  Bill Paxton & The MESA Team
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful,
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************

      module run_star_extras

      use star_lib
      use star_def
      use const_def
      use math_lib
      use num_lib

      implicit none

      ! these routines are called by the standard run_star check_model
      contains

        subroutine extras_controls(id, ierr)
           integer, intent(in) :: id
           integer, intent(out) :: ierr
           type (star_info), pointer :: s

           integer::narg,cptArg
           character(len=256) ::name
           character(len=256) ::logsDir,z_ini_s,Dmix_s,M_ini_s, f1_s
           real(16)::Dmix, M_ini, f1

           real(dp) :: tau_MS_sec, tau_MS_yrs, R_star, Dmin_min

           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return

           ! this is the place to set any procedure pointers you want to change
           ! e.g., other_wind, other_mixing, other_energy  (see star_data.inc)


           ! the extras functions in this file will not be called
           ! unless you set their function pointers as done below.
           ! otherwise we use a null_ version which does nothing (except warn).

           s% extras_startup => extras_startup
           s% extras_start_step => extras_start_step
           s% extras_check_model => extras_check_model
           s% extras_finish_step => extras_finish_step
           s% extras_after_evolve => extras_after_evolve
           s% how_many_extra_history_columns => how_many_extra_history_columns
           s% data_for_extra_history_columns => data_for_extra_history_columns
           s% how_many_extra_profile_columns => how_many_extra_profile_columns
           s% data_for_extra_profile_columns => data_for_extra_profile_columns

           s% how_many_extra_history_header_items => how_many_extra_history_header_items
           s% data_for_extra_history_header_items => data_for_extra_history_header_items
           s% how_many_extra_profile_header_items => how_many_extra_profile_header_items
           s% data_for_extra_profile_header_items => data_for_extra_profile_header_items


           s% other_mesh_delta_coeff_factor => comp_brunt_other_mesh_delta_coeff_factor


           !Check if arguments are found
           narg=command_argument_count()
           write(*,*)'Number of CL-Args: ',narg

           if(narg>0)then
               !loop across options
               do cptArg=1,narg
                   call get_command_argument(cptArg,name)
                       select case(adjustl(name))

                       ! ------ cases for varying parameters ---------------------------
                           case("--logsdir")
                               call get_command_argument(cptArg+1,logsDir)
                               write(*,*)'LOGS DIR: ',logsDir
                               s% log_directory = logsDir
                           case("--initial_mass")
                               call get_command_argument(cptArg+1,M_ini_s)
                               read(M_ini_s,*)M_ini
                               write(*,*)'Initial Mass: ',M_ini
                               s% initial_mass = M_ini
                           case("--f1")
                               call get_command_argument(cptArg+1,f1_s)
                               read(f1_s,*)f1
                               s% overshoot_f = f1 + s% overshoot_f0

                       ! ------ cases for varying CBM ------------------------------------
                           case("--exponential")
                               s% overshoot_scheme = 'exponential'
                           case("--step")
                                 s% overshoot_scheme = 'step'
                       end select
               end do
           endif

		   ! Sets a small amount of envelope mixing to obtain smoother profiles. Adjusts
		   ! the min_D_mix value based on the stellar mass to avoid significant impact on
		   ! the stellar evolution.
           tau_MS_yrs = 10**10 * s% initial_mass**(-2.5)
           tau_MS_sec = tau_MS_yrs * 60 * 60 * 24* 365
           R_star = s% initial_mass**0.57
           Dmin_min = (R_star*Rsun)**2./tau_MS_sec
           s% min_D_mix = 10**((log10(Dmin_min)-4)/2.)

        end subroutine extras_controls


        subroutine comp_brunt_other_mesh_delta_coeff_factor(id, ierr)
           integer, intent(in) :: id
           integer, intent(out) :: ierr
           type (star_info), pointer :: s
           integer :: k, k_min, k_max
           real(dp) :: max_brunt_N2_composition_term
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           
           ! This subroutine is used to increase the mesh resolution in and around the bump
           ! in the Brunt-Vaisala frequency

           do k = 1, s% nz
             s% mesh_delta_coeff_factor(k) = 0.8d0
           end do

           if (.not. s% doing_relax) then
               max_brunt_N2_composition_term = 0
               do k = 1, s% nz
                  if ((s% brunt_N2_composition_term(k) .gt. max_brunt_N2_composition_term) &
                        .and. (s% q(k) .lt. 0.9)) then
                    max_brunt_N2_composition_term = s% brunt_N2_composition_term(k)
                  endif
               enddo


               k_min = 10d5
               k_max = 0
               do k = 1, s% nz
                 if (s% brunt_N2_composition_term(k) / max_brunt_N2_composition_term .gt. 0.01) then

                    if (k .lt. k_min) then
                        k_min = k
                    endif
                    if (k .gt. k_max) then
                        k_max = k
                    endif
                 endif
               end do

               do k = 1, s% nz
                 if (s% mixing_type(k) .eq. 2) then
                   if (k .gt. k_max) then
                       k_max = k
                   endif
                 endif
               end do


               if (k_min-20 > 0) then
                 k_min = k_min-20
               else
                 k_min = 1
               endif

               if (k_max+50 < s% nz) then
                 k_max = k_max+50
               else
                 k_max = s% nz
               endif

               do k = k_min, k_max
                  s% mesh_delta_coeff_factor(k) = 0.05d0
               enddo

         endif



        end subroutine comp_brunt_other_mesh_delta_coeff_factor


        subroutine extras_startup(id, restart, ierr)
           integer, intent(in) :: id
           logical, intent(in) :: restart
           integer, intent(out) :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
        end subroutine extras_startup


        integer function extras_start_step(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           extras_start_step = 0
        end function extras_start_step


        ! returns either keep_going, retry, or terminate.
        integer function extras_check_model(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           extras_check_model = keep_going
           if (.false. .and. s% star_mass_h1 < 0.35d0) then
              ! stop when star hydrogen mass drops to specified level
              extras_check_model = terminate
              write(*, *) 'have reached desired hydrogen mass'
              return
           end if


           ! if you want to check multiple conditions, it can be useful
           ! to set a different termination code depending on which
           ! condition was triggered.  MESA provides 9 customizeable
           ! termination codes, named t_xtra1 .. t_xtra9.  You can
           ! customize the messages that will be printed upon exit by
           ! setting the corresponding termination_code_str value.
           ! termination_code_str(t_xtra1) = 'my termination condition'

           ! by default, indicate where (in the code) MESA terminated
           if (extras_check_model == terminate) s% termination_code = t_extras_check_model
        end function extras_check_model


        integer function how_many_extra_history_columns(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           how_many_extra_history_columns = 0
        end function how_many_extra_history_columns


        subroutine data_for_extra_history_columns(id, n, names, vals, ierr)
           integer, intent(in) :: id, n
           character (len=maxlen_history_column_name) :: names(n)
           real(dp) :: vals(n)
           integer, intent(out) :: ierr
           type (star_info), pointer :: s
           integer :: k
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return

           ! note: do NOT add the extras names to history_columns.list
           ! the history_columns.list is only for the built-in history column options.
           ! it must not include the new column names you are adding here.

        end subroutine data_for_extra_history_columns


        integer function how_many_extra_profile_columns(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           how_many_extra_profile_columns = 1
        end function how_many_extra_profile_columns


        subroutine data_for_extra_profile_columns(id, n, nz, names, vals, ierr)
           integer, intent(in) :: id, n, nz
           character (len=maxlen_profile_column_name) :: names(n)
           real(dp) :: vals(nz,n)
           integer, intent(out) :: ierr
           type (star_info), pointer :: s
           integer :: k, k_max
           real(dp) :: dmu_dm, max_brunt_N2_composition_term
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return

           ! note: do NOT add the extra names to profile_columns.list
           ! the profile_columns.list is only for the built-in profile column options.
           ! it must not include the new column names you are adding here.

           ! here is an example for adding a profile column
           !if (n /= 1) stop 'data_for_extra_profile_columns'
           !names(1) = 'beta'
           !do k = 1, nz
           !   vals(k,1) = s% Pgas(k)/s% P(k)
           !end do


           max_brunt_N2_composition_term = 0
           !k_max = -1
           do k = 1, nz
              if ((s% brunt_N2_composition_term(k) .gt. max_brunt_N2_composition_term) &
                    .and. (s% q(k) .lt. 0.9)) then
                max_brunt_N2_composition_term = s% brunt_N2_composition_term(k)
                k_max = k
              endif
           enddo

           names(1) = 'brunt_N2_composition_term_div_surface'
           do k = 1, nz
             vals(k,1) = s% brunt_N2_composition_term(k) / max_brunt_N2_composition_term
           end do


        end subroutine data_for_extra_profile_columns


        integer function how_many_extra_history_header_items(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           how_many_extra_history_header_items = 0
        end function how_many_extra_history_header_items


        subroutine data_for_extra_history_header_items(id, n, names, vals, ierr)
           integer, intent(in) :: id, n
           character (len=maxlen_history_column_name) :: names(n)
           real(dp) :: vals(n)
           type(star_info), pointer :: s
           integer, intent(out) :: ierr
           ierr = 0
           call star_ptr(id,s,ierr)
           if(ierr/=0) return

           ! here is an example for adding an extra history header item
           ! also set how_many_extra_history_header_items
           ! names(1) = 'mixing_length_alpha'
           ! vals(1) = s% mixing_length_alpha

        end subroutine data_for_extra_history_header_items


        integer function how_many_extra_profile_header_items(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           how_many_extra_profile_header_items = 0
        end function how_many_extra_profile_header_items


        subroutine data_for_extra_profile_header_items(id, n, names, vals, ierr)
           integer, intent(in) :: id, n
           character (len=maxlen_profile_column_name) :: names(n)
           real(dp) :: vals(n)
           type(star_info), pointer :: s
           integer, intent(out) :: ierr
           ierr = 0
           call star_ptr(id,s,ierr)
           if(ierr/=0) return

           ! here is an example for adding an extra profile header item
           ! also set how_many_extra_profile_header_items
           ! names(1) = 'mixing_length_alpha'
           ! vals(1) = s% mixing_length_alpha

        end subroutine data_for_extra_profile_header_items


        ! returns either keep_going or terminate.
        ! note: cannot request retry; extras_check_model can do that.
        integer function extras_finish_step(id)
           integer, intent(in) :: id
           integer :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
           extras_finish_step = keep_going

           if (.not. s% L_nuc_burn_total / s% L_phot < 0.999d0) then
              s% max_years_for_timestep = s% nuc_timescale*2.0d-3
          !    !s% max_dq = 1d-3
           end if

           ! to save a profile,
              ! s% need_to_save_profiles_now = .true.
           ! to update the star log,
              ! s% need_to_update_history_now = .true.

           ! see extras_check_model for information about custom termination codes
           ! by default, indicate where (in the code) MESA terminated
           if (extras_finish_step == terminate) s% termination_code = t_extras_finish_step
        end function extras_finish_step


        subroutine extras_after_evolve(id, ierr)
           integer, intent(in) :: id
           integer, intent(out) :: ierr
           type (star_info), pointer :: s
           ierr = 0
           call star_ptr(id, s, ierr)
           if (ierr /= 0) return
        end subroutine extras_after_evolve


      end module run_star_extras
